# -*- python -*-
# $RCSfile: progressbarGUI.py,v $
# $Revision: 1.38 $
# $Author: vrc $
# $Date: 2007-09-26 22:02:30 $

# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

from ooflib.SWIG.common import switchboard
from ooflib.SWIG.common.IO import progressbar
from ooflib.common import debug
from ooflib.common import guitop
from ooflib.common.IO import progressbar_delay
from ooflib.common.IO import reporter
from ooflib.common.IO import threadmanager
from ooflib.common.IO.GUI import activityViewer
from ooflib.common.IO.GUI import gtklogger
from ooflib.common.IO.GUI import gtkutils
import gobject
import gtk

#########################################################################
#########################################################################
## The following is the Graphic progress bar class and sends messages to
## GTK.
#########################################################################
#########################################################################

class GUIProgressBar (progressbar.ProgressBarRepresentation):
    idcounter = 0 
    def __init__(self, progressBar):
        debug.mainthreadTest()
        progressbar.ProgressBarRepresentation.__init__(self, progressBar)
        # progressBar is actually a ProgressObject instance
        self.button_state = 'stop'

        self.gtk = gtk.Frame()
        self.gtk.set_shadow_type(gtk.SHADOW_IN)
        vbox = gtk.VBox(spacing=2)
        self.gtk.add(vbox)

        ## Create progress line
        self.progressline = gtk.HBox()
        vbox.pack_start(self.progressline, expand=1, fill=1, padding=0)
##        self.progressline.connect('destroy', self.destroyCB)

        ## Create the progress bar
        self.gtkprogressbar = gtk.ProgressBar()
        self.gtkprogressbar.set_orientation(gtk.PROGRESS_LEFT_TO_RIGHT)
        self.progressline.pack_start(self.gtkprogressbar,
                                     expand=1, fill=1, padding=2)

        ## Stop button lives here
        self.stopbutton = gtkutils.stockButton(gtk.STOCK_STOP, "Stop")
        gtklogger.setWidgetName(self.stopbutton, "Stop%d" % self.idcounter)
        self.idcounter += 1
        gtklogger.connect(self.stopbutton, "clicked", self.stopButtonCB)
        self.progressline.pack_start(self.stopbutton,
                                     expand=0, fill=0, padding=2)

        ## Create the bottom label, if any
        self.name = self.get_name() 
        if self.name:
            scrllwd = gtk.ScrolledWindow()
            scrllwd.set_shadow_type(gtk.SHADOW_ETCHED_OUT)
            scrllwd.set_policy(gtk.POLICY_AUTOMATIC,
                                    gtk.POLICY_AUTOMATIC)
            vbox.add(scrllwd) 
            label_bottom = gtk.TextView()
            label_bottom.set_editable(False)
            label_bottom.set_cursor_visible(False)
            label_bottom.set_wrap_mode(gtk.WRAP_WORD)
            scrllwd.add(label_bottom)
            label_bottom.get_buffer().set_text(self.name)

        # Schedule the bar for periodic updates via the gtk timeout
        # events.  The time is specified in milliseconds.
        self.timeout_id = gobject.timeout_add(progressbar_delay.minimum_time(),
                                              self.updateGUI)
        
    def get_name(self):
        return self.progressbar.get_name()

    def destroy(self):
        # Called by stop button callback, and when thread finishes.
        debug.mainthreadTest()
        self.disconnect()
        self.gtk.destroy()
        activityViewer.sensitize()

    def disconnect(self):
        debug.mainthreadTest()
        gobject.source_remove(self.timeout_id) # stop calling updateGUI
        threadmanager.threadManager.removeProgressBar(self.progressbar)
        activityViewer.activityViewer.dismissgtkbar(self)
        threadmanager.threadManager.removeWorker(self.progressbar.Worker)
        self.progressbar.unregisterWorker() ## remove worker reference from pbar
        
    def show(self):
        debug.mainthreadTest()
        self.gtk.show_all()

    def get_success(self):
        return self.progressbar.get_success()
    
    def stopButtonCB(self, *args):
        if self.button_state == 'stop': # first click
            if self.progressbar.get_success() == progressbar.INCOMPLETE:
                self.progressbar.stop_it()
                if threadmanager.threadManager.AutoDismiss:
                    self.destroy()
                else:
                    # Change the "Stop" button to a "Dismiss" button.
                    self.create_dismiss_button()
        else:                           # second click
            self.destroy()

    def updateGUI(self):                # timeout callback
        debug.mainthreadTest()
        # are we there yet?
        done = self.progressbar.get_success() != progressbar.INCOMPLETE
        # Update the text inside the progress bar.
        msg = self.progressbar.get_message()
        if msg is not None :
            self.gtkprogressbar.set_text(msg)
        # Update the progress bar itself.
        if self.type == "active":
            if not done:
                self.gtkprogressbar.pulse()
        else:
            value = self.progressbar.get_progress()
            if value > 1.0 or done:     # Check for round-off and completion.
                value = 1.0
            self.gtkprogressbar.set_fraction(value)

        if done:
            if threadmanager.threadManager.AutoDismiss and done:
                self.destroy()
            else:
                self.create_dismiss_button()
        activityViewer.sensitize()
        return not done                 # True ==> keep calling the function

    def create_dismiss_button(self):
        debug.mainthreadTest()
        label = gtkutils.findChild(gtk.Label, self.stopbutton)
        label.set_text("Dismiss")
        image = gtkutils.findChild(gtk.Image, self.stopbutton)
        image.set_from_stock(gtk.STOCK_CANCEL, gtk.ICON_SIZE_BUTTON)
        self.button_state = 'dismiss'


def set_ProgressBarRepresentation(pbar):
    ## overrides what it is done in progressbar.spy
    ## argument is a ProgressBar
    ## Creates a progressbar representation, based on the mode
    ## (text or graphic) that OOF is currently running.
    if guitop.top():
        return GUIProgressBar(pbar)
    else:
        return progressbar.TextProgressBar(pbar)

progressbar.set_ProgressBarRepresentation = set_ProgressBarRepresentation
