# -*- python -*-
# $RCSfile: excepthook.py,v $
# $Revision: 1.10.8.2 $
# $Author: langer $
# $Date: 2008-08-18 21:19:04 $


# This software was produced by NIST, an agency of the U.S. government,
# and by statute is not subject to copyright in the United States.
# Recipients of this software assume all responsibilities associated
# with its operation, modification and maintenance. However, to
# facilitate maintenance we ask that before distributing modified
# versions of this software, you first contact the authors at
# oof_manager@ctcms.nist.gov. 

# A module for managing the Python exception hook.

import sys, traceback
from ooflib.common import debug

def _printTraceBack(e_type, e_value, tblist):
    for line in traceback.format_exception_only(e_type, e_value):
        print line,
    for line in traceback.format_list(tblist):
        print line,

# printTraceBack is overridden by reporter_GUI.py, so that in GUI mode
# it brings up a pop-up window.  The _OOFexcepthook function is not
# overridden by the GUI, nor does assign_excepthook get re-called.
printTraceBack = _printTraceBack

def _OOFexceptHook(e_type, e_value, tback):
    tblist = traceback.extract_tb(tback)
    # In debug mode, always dump the traceback to the terminal, but
    # don't do it twice.
    if debug.debug() and printTraceBack != _printTraceBack:
        _printTraceBack(e_type, e_value, tblist)
    printTraceBack(e_type, e_value, tblist)
    # Now that we've handled the exception, clear it.  The system's
    # exception data keeps a reference to the local dictionary of the
    # frame in which the exception occurred, and this can prevent
    # garbage collection.  TODO: One can imagine circumstances in
    # which this isn't the right thing to do, but those circumstances
    # should probably assign a new excepthook function.
    sys.exc_clear()

old_excepthook = None

### Should be called zero or one times.
def assign_excepthook(new_excepthook):
    global old_excepthook
    assert old_excepthook is None
    old_excepthook = sys.excepthook
    sys.excepthook = new_excepthook

def restore_excepthook():
    global old_excepthook
    assert old_excepthook is not None
    sys.excepthook = old_excepthook

assign_excepthook(_OOFexceptHook)
